#include <stdlib.h>
#include <stdio.h>
#include "kernel.cu"

#define CUDA_ERRCHK(fn) { __gpucheck((fn), __FILE__, __LINE__); }

inline cudaError_t __gpucheck(cudaError_t code, const char *file, int line) {
    if (code != cudaSuccess) {
        fprintf(stderr, "CUDA runtime error [%s:%d]: %s\n", file, line, cudaGetErrorString(code));
        exit(-1);
    }
    return code;
}

__global__
void mulAll_kernel(int *out, int *in, int n, size_t size) {
    for (int i = blockIdx.x * blockDim.x + threadIdx.x;
         i < size;
         i += blockDim.x + gridDim.x) {
        out[i] = in[i] * n;
    }
}

extern "C" {
void mulAll(int *out, const int *input, int n, size_t size) {
    int *out_d, *in_d, i_d, size_d;

    CUDA_ERRCHK(cudaMalloc((void **) &in_d, sizeof(int) * size));
    CUDA_ERRCHK(cudaMemcpy(in_d, input, sizeof(int) * size, cudaMemcpyHostToDevice));

    CUDA_ERRCHK(cudaMalloc((void **) &out_d, sizeof(int) * size));
    mulAll_kernel << < 32, 32 >> > (out_d, in_d, n, size);
    CUDA_ERRCHK(cudaMemcpy(out, out_d, sizeof(int) * size, cudaMemcpyDeviceToHost));

    CUDA_ERRCHK(cudaFree(in_d));
    CUDA_ERRCHK(cudaFree(out_d));
}
}
